"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printBundlephobiaStats = exports.calculateBundlephobiaStats = void 0;
const package_build_stats_1 = require("package-build-stats");
const pretty_bytes_1 = __importDefault(require("pretty-bytes"));
const chalk_1 = __importDefault(require("chalk"));
const silenceConsole = () => {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    const noop = () => { };
    const consoleFunctions = {};
    let key;
    for (key in console) {
        if (typeof console[key] === 'function') {
            consoleFunctions[key] = console[key];
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore basically intentionally violating type safe code here to silence the console
            console[key] = noop;
        }
    }
    return consoleFunctions;
};
const reinstateConsole = (consoleFunctions) => {
    Object.entries(consoleFunctions).forEach(([key, func]) => {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        console[key] = func;
    });
};
const calculateBundlephobiaStats = async ({ cwd, }) => {
    const consoleFunctions = silenceConsole();
    try {
        const packageStats = await package_build_stats_1.getPackageStats(cwd);
        reinstateConsole(consoleFunctions);
        return packageStats;
    }
    catch (error) {
        reinstateConsole(consoleFunctions);
        throw error;
    }
};
exports.calculateBundlephobiaStats = calculateBundlephobiaStats;
const printBundlephobiaStats = (packageStats) => {
    // hex #00de6d is to match color used by progress estimator for ✓
    // https://github.com/bvaughn/progress-estimator/blob/master/src/theme.js#L18
    const logLine = (text, emptyLineAbove) => {
        console.log(`${emptyLineAbove ? '\n' : ''}${chalk_1.default.hex('#00de6d')('➜')} ${text}`);
    };
    logLine(`Minified size: ${chalk_1.default.bold(pretty_bytes_1.default(packageStats.size))}`, true);
    logLine(`Minified and gzipped size: ${chalk_1.default.bold(pretty_bytes_1.default(packageStats.gzip))}`);
    if (Array.isArray(packageStats.dependencySizes)) {
        const totalDependencySizes = packageStats.dependencySizes.reduce((total, value) => {
            return total + value.approximateSize;
        }, 0);
        const sortedDependencies = packageStats.dependencySizes.sort((a, b) => b.approximateSize - a.approximateSize);
        sortedDependencies.forEach((dependency) => {
            const dependencyPercent = (dependency.approximateSize / totalDependencySizes) * 100;
            const roundedDependencyPercent = dependencyPercent === 100
                ? dependencyPercent
                : dependencyPercent.toFixed(1);
            logLine(`${roundedDependencyPercent}% ${dependency.name}`);
        });
    }
};
exports.printBundlephobiaStats = printBundlephobiaStats;
/////////////////////////////////////
//# sourceMappingURL=bundlephobiaStats.js.map