"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkInvariantsAndGetConfiguration = void 0;
const path_1 = require("path");
const fs = __importStar(require("fs-extra"));
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const utils_1 = require("./utils");
const errorUtils_1 = require("./errorUtils");
const checkInvariantsAndGetConfiguration = async ({ args, cwd, }) => {
    errorUtils_1.invariant(args[0] === 'build' || args[0] === 'watch', `rollpkg requires a "build" or "watch" command with no arguments, received: "${args.join(' ')}"`);
    const watchMode = args[0] === 'watch';
    let tsconfigPath;
    const tsconfigOptionIdx = args.indexOf('--tsconfig');
    if (tsconfigOptionIdx !== -1) {
        tsconfigPath = args[tsconfigOptionIdx + 1];
        try {
            await fs.readFile(tsconfigPath, 'utf8');
        }
        catch (e) {
            throw new Error(`Cannot read tsconfig at ${tsconfigPath}`);
        }
    }
    const addUmdBuild = args.includes('--addUmdBuild');
    const includeBundlephobiaStats = !args.includes('--noStats');
    let pkgJson;
    try {
        const readPkgJson = await fs.readFile(path_1.resolve(cwd, 'package.json'), 'utf8');
        pkgJson = JSON.parse(readPkgJson);
    }
    catch {
        throw Error(`Cannot read package.json at ${path_1.resolve(cwd, 'package.json')}`);
    }
    errorUtils_1.invariant(typeof pkgJson.name === 'string', `"name" field is required in package.json and needs to be a string, value found: ${pkgJson.name}`);
    // type cast as string because previous invariant check makes this code
    // unreachable if pkgJson.name is not a string, which TS doesn't understand
    const pkgJsonName = pkgJson.name;
    const isValidName = validate_npm_package_name_1.default(pkgJsonName);
    errorUtils_1.invariant(isValidName.validForNewPackages, `Invalid npm package name, see https://www.npmjs.com/package/validate-npm-package-name\n${isValidName.errors?.length
        ? isValidName.errors
            ?.map((npmError) => `Npm Error: ${npmError}`)
            .join('\n')
        : ''}`);
    const kebabCasePkgName = utils_1.convertPkgNameToKebabCase(pkgJsonName);
    const mainShouldBe = `dist/${kebabCasePkgName}.cjs.js`;
    const moduleShouldBe = `dist/${kebabCasePkgName}.esm.js`;
    const typesShouldBe = `dist/index.d.ts`;
    errorUtils_1.invariant(pkgJson.main === mainShouldBe, `The value of "main" in package.json needs to be "${mainShouldBe}", value found: "${pkgJson.main}"`);
    errorUtils_1.invariant(pkgJson.module === moduleShouldBe, `The value of "module" in package.json needs to be "${moduleShouldBe}", value found: "${pkgJson.module}"`);
    errorUtils_1.invariant(!pkgJson.types || pkgJson.types === typesShouldBe, `The value of "types" in package.json needs to be "${typesShouldBe}", value found: "${pkgJson.types}"`);
    errorUtils_1.invariant(typeof pkgJson.sideEffects === 'boolean', `"sideEffects" field is required in package.json and needs to be a boolean, value found: ${pkgJson.sideEffects}`);
    // type cast as boolean because previous invariant check makes this code
    // unreachable if pkgJson.name is not a string, which TS doesn't understand
    const pkgJsonSideEffects = pkgJson.sideEffects;
    const pkgJsonDependencyKeys = pkgJson.dependencies
        ? Object.keys(pkgJson.dependencies)
        : [];
    const pkgJsonPeerDependencyKeys = pkgJson.peerDependencies
        ? Object.keys(pkgJson.peerDependencies)
        : [];
    errorUtils_1.invariant(!pkgJson.umdGlobalDependencies ||
        (typeof pkgJson.umdGlobalDependencies === 'object' &&
            !Array.isArray(pkgJson.umdGlobalDependencies) &&
            Object.values(pkgJson.umdGlobalDependencies).every((value) => typeof value === 'string')), 'If "umdGlobalDependencies" is specified in package.json, it needs to be an object of the form { "package-name": "GlobalName" }, for example { "react-dom": "ReactDOM" }');
    const pkgJsonUmdGlobalDependencies = pkgJson.umdGlobalDependencies;
    const srcDir = path_1.resolve(cwd, 'src');
    const indexTsExists = await utils_1.fileExists(srcDir, 'index.ts');
    const indexTsxExists = await utils_1.fileExists(srcDir, 'index.tsx');
    errorUtils_1.invariant(!(indexTsExists && indexTsxExists), `Cannot have both index.ts and index.tsx files in ${srcDir}`);
    errorUtils_1.invariant(indexTsExists || indexTsxExists, `Cannot find index.ts or index.tsx entry file in ${srcDir}`);
    const entryFile = path_1.resolve(srcDir, indexTsExists ? 'index.ts' : 'index.tsx');
    return {
        watchMode,
        tsconfigPath,
        addUmdBuild,
        includeBundlephobiaStats,
        pkgJsonName,
        kebabCasePkgName,
        entryFile,
        pkgJsonSideEffects,
        pkgJsonDependencyKeys,
        pkgJsonPeerDependencyKeys,
        pkgJsonUmdGlobalDependencies,
    };
};
exports.checkInvariantsAndGetConfiguration = checkInvariantsAndGetConfiguration;
//# sourceMappingURL=configureRollpkg.js.map