"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeBundles = exports.createBundles = exports.rollupWatch = exports.createRollupConfig = void 0;
const path_1 = require("path");
const fs = __importStar(require("fs-extra"));
const rollup_1 = require("rollup");
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_commonjs_1 = __importDefault(require("@rollup/plugin-commonjs"));
const plugin_json_1 = __importDefault(require("@rollup/plugin-json"));
const rollup_plugin_typescript2_1 = __importDefault(require("rollup-plugin-typescript2"));
const plugin_replace_1 = __importDefault(require("@rollup/plugin-replace"));
const rollup_plugin_sourcemaps_1 = __importDefault(require("rollup-plugin-sourcemaps"));
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const utils_1 = require("./utils");
const errorUtils_1 = require("./errorUtils");
const createRollupConfig = ({ tsconfigPath, addUmdBuild, kebabCasePkgName, pkgJsonSideEffects, pkgJsonPeerDependencyKeys, pkgJsonUmdGlobalDependencies, }) => {
    const buildPluginsDefault = [
        plugin_node_resolve_1.default({ preferBuiltins: true }),
        plugin_commonjs_1.default(),
        plugin_json_1.default(),
        rollup_plugin_typescript2_1.default({
            // verbosity: 3, // use to debug
            tsconfigDefaults: {
                compilerOptions: {
                    target: 'ES2018',
                    // types: [] eliminates global type pollution in builds
                    // and ensures that the only types allowed
                    // are explicitly set in tsconfig or are imported into source files
                    types: [],
                    // generate *.d.ts files by default
                    declaration: true,
                    // enforces convention that all files to be included in the build are in the src directory
                    // this doesn't prevent other non-build files like *.mock.ts from being outside the src directory
                    rootDir: './src',
                },
                // include rollpkg types that stub process.env.NODE_ENV and __DEV__
                // so they can be used without polluting the global type space with all node types etc
                include: ['src', './node_modules/rollpkg/configs/types'],
                // exclude tests, mocks and snapshots
                exclude: [
                    '**/__tests__',
                    '**/__mocks__',
                    '**/__snapshots__',
                    '**/*.test.*',
                    '**/*.spec.*',
                    '**/*.mock.*',
                ],
            },
            tsconfig: tsconfigPath,
            tsconfigOverride: {
                compilerOptions: {
                    // rollup requires module to be es2015 or esnext
                    module: 'ESNext',
                    // always generate source maps which are used by rollup to create the actual source map
                    // without this rollup creates blank source maps
                    // note that the tsconfig "inlineSources" option has no effect on how rollup generates source maps
                    // as rollup has it's own inline sources option, "sourcemapExcludeSources" which defaults to false
                    sourceMap: true,
                },
            },
            include: ['**/*.ts+(|x)', '**/*.js+(|x)'],
        }),
        rollup_plugin_sourcemaps_1.default(),
        plugin_replace_1.default({
            __DEV__: "process.env.NODE_ENV !== 'production'",
            preventAssignment: true,
        }),
    ];
    const buildPluginsWithNodeEnvDevelopment = [
        ...buildPluginsDefault,
        plugin_replace_1.default({
            'process.env.NODE_ENV': JSON.stringify('development'),
            preventAssignment: true,
        }),
    ];
    const buildPluginsWithNodeEnvProduction = [
        ...buildPluginsDefault,
        plugin_replace_1.default({
            'process.env.NODE_ENV': JSON.stringify('production'),
            preventAssignment: true,
        }),
    ];
    const outputPluginsDefault = [];
    const outputPluginsProduction = [
        rollup_plugin_terser_1.terser({ format: { comments: false } }),
    ];
    const treeshakeOptions = {
        annotations: true,
        moduleSideEffects: pkgJsonSideEffects,
        propertyReadSideEffects: pkgJsonSideEffects,
        tryCatchDeoptimization: pkgJsonSideEffects,
        unknownGlobalSideEffects: pkgJsonSideEffects,
    };
    let umdExternalDependencies;
    let umdDependencyGlobals;
    let umdNameForPkg;
    if (addUmdBuild) {
        // umdExternalDependencies is an array of external module ids that rollup
        // will not include in the build
        umdExternalDependencies = pkgJsonUmdGlobalDependencies
            ? Object.keys(pkgJsonUmdGlobalDependencies)
            : pkgJsonPeerDependencyKeys;
        // umdDependencyGlobals is an object where the keys are the module ids (the umdExternalDependencies list)
        // and the values are what those modules will be available in the global scope as
        // for example { 'react-dom': 'ReactDOM' } because react-dom will be available on the window as ReactDOM
        if (pkgJsonUmdGlobalDependencies) {
            umdDependencyGlobals = pkgJsonUmdGlobalDependencies;
        }
        else {
            pkgJsonPeerDependencyKeys.forEach((peerDep) => {
                umdDependencyGlobals = {};
                umdDependencyGlobals[peerDep] = utils_1.convertKebabCaseToPascalCase(utils_1.convertPkgNameToKebabCase(peerDep));
            });
        }
        umdNameForPkg = utils_1.convertKebabCaseToPascalCase(kebabCasePkgName);
    }
    return {
        buildPluginsDefault,
        buildPluginsWithNodeEnvDevelopment,
        buildPluginsWithNodeEnvProduction,
        outputPluginsDefault,
        outputPluginsProduction,
        treeshakeOptions,
        umdNameForPkg,
        umdExternalDependencies,
        umdDependencyGlobals,
    };
};
exports.createRollupConfig = createRollupConfig;
const rollupWatch = ({ kebabCasePkgName, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, entryFile, treeshakeOptions, buildPluginsDefault, outputPluginsDefault, }) => {
    // exit 0 from watch mode on ctrl-c (SIGINT) etc so can chain npm scripts: rollpkg watch && ...
    // the ... will run after rollpkg watch only if it exits 0
    const exitWatch = () => {
        utils_1.clearLine();
        console.log('ROLLPKG WATCH END 👀👋', '\n');
        process.exit(0);
    };
    process.on('SIGINT', exitWatch);
    process.on('SIGTERM', exitWatch);
    process.on('SIGBREAK', exitWatch);
    const esmWatchOptions = {
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsDefault,
        output: {
            file: `dist/${kebabCasePkgName}.esm.js`,
            format: 'esm',
            sourcemap: true,
            plugins: outputPluginsDefault,
        },
    };
    // in watch mode only create esm build
    const watcher = rollup_1.watch(esmWatchOptions);
    const earth = ['🌎', '🌏', '🌍'];
    let currentEarth = 2;
    const rotateEarth = () => {
        if (currentEarth === 2)
            currentEarth = 0;
        else {
            currentEarth = currentEarth + 1;
        }
        return earth[currentEarth];
    };
    watcher.on('event', (event) => {
        switch (event.code) {
            case 'START':
                break;
            case 'BUNDLE_START':
                utils_1.clearConsole();
                console.log(`${rotateEarth()} Rollpkg building...`, '\n');
                break;
            case 'BUNDLE_END':
                utils_1.clearConsole();
                console.log(`${earth[currentEarth]} Rollpkg build successful!`, '\n');
                console.log('Watching for changes...', '\n');
                break;
            case 'END':
                break;
            case 'ERROR':
                utils_1.clearConsole();
                if (event.error.plugin === 'rpt2') {
                    errorUtils_1.logTsError({ message: event.error.message });
                }
                else {
                    errorUtils_1.logError({ fullError: event.error });
                }
                console.log('Watching for changes...', '\n');
                break;
        }
    });
};
exports.rollupWatch = rollupWatch;
const createBundles = ({ entryFile, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, umdExternalDependencies, treeshakeOptions, buildPluginsDefault, buildPluginsWithNodeEnvDevelopment, buildPluginsWithNodeEnvProduction, addUmdBuild, }) => Promise.all([
    // default bundle - used for esm and cjs dev builds
    rollup_1.rollup({
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsDefault,
    }),
    // cjs production build
    rollup_1.rollup({
        external: [...pkgJsonDependencyKeys, ...pkgJsonPeerDependencyKeys],
        input: entryFile,
        treeshake: treeshakeOptions,
        plugins: buildPluginsWithNodeEnvProduction,
    }),
    // umd development build
    addUmdBuild
        ? rollup_1.rollup({
            external: umdExternalDependencies,
            input: entryFile,
            treeshake: treeshakeOptions,
            plugins: buildPluginsWithNodeEnvDevelopment,
        })
        : undefined,
    // umd production build
    addUmdBuild
        ? rollup_1.rollup({
            external: umdExternalDependencies,
            input: entryFile,
            treeshake: treeshakeOptions,
            plugins: buildPluginsWithNodeEnvProduction,
        })
        : undefined,
]);
exports.createBundles = createBundles;
const writeBundles = ({ cwd, kebabCasePkgName, bundleDefault, bundleCjsProd, bundleUmdDev, bundleUmdProd, outputPluginsDefault, outputPluginsProduction, umdNameForPkg, umdDependencyGlobals, }) => {
    // prettier-ignore
    const cjsEntryContent = `'use strict';

if (process.env.NODE_ENV === 'production') {
  module.exports = require('./${kebabCasePkgName}.cjs.production.js');
} else {
  module.exports = require('./${kebabCasePkgName}.cjs.development.js');
}
`;
    return Promise.all([
        // esm build
        bundleDefault.write({
            file: `dist/${kebabCasePkgName}.esm.js`,
            format: 'esm',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsDefault,
        }),
        // cjs development build
        bundleDefault.write({
            file: `dist/${kebabCasePkgName}.cjs.development.js`,
            format: 'cjs',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsDefault,
        }),
        // cjs production build
        bundleCjsProd.write({
            file: `dist/${kebabCasePkgName}.cjs.production.js`,
            format: 'cjs',
            sourcemap: true,
            sourcemapExcludeSources: false,
            plugins: outputPluginsProduction,
        }),
        // cjs entry file
        fs.writeFile(path_1.resolve(cwd, 'dist', `${kebabCasePkgName}.cjs.js`), cjsEntryContent, 'utf8'),
        bundleUmdDev
            ? bundleUmdDev.write({
                file: `dist/${kebabCasePkgName}.umd.development.js`,
                format: 'umd',
                name: umdNameForPkg,
                globals: umdDependencyGlobals,
                sourcemap: true,
                sourcemapExcludeSources: false,
                plugins: outputPluginsDefault,
            })
            : undefined,
        bundleUmdProd
            ? bundleUmdProd.write({
                file: `dist/${kebabCasePkgName}.umd.production.js`,
                format: 'umd',
                name: umdNameForPkg,
                globals: umdDependencyGlobals,
                sourcemap: true,
                sourcemapExcludeSources: false,
                plugins: outputPluginsProduction,
            })
            : undefined,
    ]);
};
exports.writeBundles = writeBundles;
/////////////////////////////////////
//# sourceMappingURL=rollupBuilds.js.map